@@ -14,17 +14,17 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
+#include <math.h>
 
 #include "ui.h"
+#include "book-view.h"
 
 
 enum {
     START_SCAN,
     STOP_SCAN,
-    ROTATE_LEFT,
-    ROTATE_RIGHT,
     SAVE,
-    PRINT,
+    EMAIL,
     QUIT,
     LAST_SIGNAL
 };
@@ -34,15 +34,26 @@ static guint signals[LAST_SIGNAL] = { 0, };
 struct SimpleScanPrivate
 {
     GConfClient *client;
+    
+    GtkBuilder *builder;
 
     GtkWidget *window;
     GtkWidget *scan_button_label, *continuous_scan_button_label, *page_label;
-    GtkWidget *actions_box;
     GtkWidget *device_combo, *mode_combo;
     GtkTreeModel *device_model, *mode_model;
-    GtkWidget *replace_pages_check;
     GtkWidget *preview_area;
     GtkWidget *zoom_scale;
+    GtkWidget *page_delete_menuitem, *crop_rotate_menuitem;
+
+    GtkWidget *authorize_dialog;
+    GtkWidget *authorize_label;
+    GtkWidget *username_entry, *password_entry;
+
+    GtkWidget *preferences_dialog;
+    GtkWidget *replace_pages_check;
+
+    BookView *book_view;
+    gboolean updating_page_menu;
 
     gchar *default_file_name;
     gboolean scanning;
@@ -78,6 +89,31 @@ ui_set_default_file_name (SimpleScan *ui, const gchar *default_file_name)
 }
 
 
+void
+ui_authorize (SimpleScan *ui, const gchar *resource, gchar **username, gchar **password)
+{
+    GString *description;
+
+    description = g_string_new ("");
+    g_string_printf (description,
+                     /* Label in authorization dialog.  '%s' is replaced with the name of the resource requesting authorization */
+                     _("Username and password required to access '%s'"),
+                     resource);
+
+    gtk_entry_set_text (GTK_ENTRY (ui->priv->username_entry), *username ? *username : "");
+    gtk_entry_set_text (GTK_ENTRY (ui->priv->password_entry), "");
+    gtk_label_set_text (GTK_LABEL (ui->priv->authorize_label), description->str);
+    g_string_free (description, TRUE);
+
+    gtk_widget_show (ui->priv->authorize_dialog);
+    gtk_dialog_run (GTK_DIALOG (ui->priv->authorize_dialog));
+    gtk_widget_hide (ui->priv->authorize_dialog);
+
+    *username = g_strdup (gtk_entry_get_text (GTK_ENTRY (ui->priv->username_entry)));
+    *password = g_strdup (gtk_entry_get_text (GTK_ENTRY (ui->priv->password_entry)));
+}
+
+
 void
 ui_mark_devices_undetected (SimpleScan *ui)
 {
@@ -182,6 +218,7 @@ get_replace_pages (SimpleScan *ui)
 }
 
 
+void scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
 scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
@@ -203,6 +240,7 @@ scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 }
 
 
+void continuous_scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
 continuous_scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
@@ -224,22 +262,226 @@ continuous_scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 }
 
 
+void preferences_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+preferences_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    gtk_window_present (GTK_WINDOW (ui->priv->preferences_dialog));
+}
+
+
+gboolean preferences_dialog_delete_event_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+gboolean
+preferences_dialog_delete_event_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    return TRUE;
+}
+
+
+void preferences_dialog_response_cb (GtkWidget *widget, gint response_id, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+preferences_dialog_response_cb (GtkWidget *widget, gint response_id, SimpleScan *ui)
+{
+    gtk_widget_hide (ui->priv->preferences_dialog);
+}
+
+
+static void
+page_selected_cb (BookView *view, Page *page, SimpleScan *ui)
+{
+    char *name = NULL;
+
+    if (page == NULL)
+        return;
+
+    ui->priv->updating_page_menu = TRUE;
+    
+    if (page_has_crop (page)) {
+        char *crop_name;
+
+        // FIXME: Make more generic
+        crop_name = page_get_named_crop (page);
+        if (crop_name) {
+            if (strcmp (crop_name, "A4") == 0)
+                name = "a4_menuitem";
+            else if (strcmp (crop_name, "A5") == 0)
+                name = "a5_menuitem";
+            else if (strcmp (crop_name, "A6") == 0)
+                name = "a6_menuitem";
+            else if (strcmp (crop_name, "letter") == 0)
+                name = "letter_menuitem";
+            else if (strcmp (crop_name, "legal") == 0)
+                name = "legal_menuitem";
+            else if (strcmp (crop_name, "4x6") == 0)
+                name = "4x6_menuitem";
+            g_free (crop_name);
+        }
+        else
+            name = "custom_crop_menuitem";
+    }
+    else
+        name = "no_crop_menuitem";
+
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (gtk_builder_get_object (ui->priv->builder, name)), TRUE);
+    
+    ui->priv->updating_page_menu = FALSE;
+}
+
+
+void rotate_left_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
 rotate_left_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
-    g_signal_emit (G_OBJECT (ui), signals[ROTATE_LEFT], 0);
+    if (ui->priv->updating_page_menu)
+        return;
+    page_rotate_left (book_view_get_selected (ui->priv->book_view));
 }
 
 
+void rotate_right_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
 rotate_right_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
-    g_signal_emit (G_OBJECT (ui), signals[ROTATE_RIGHT], 0);
+    if (ui->priv->updating_page_menu)
+        return;
+    page_rotate_right (book_view_get_selected (ui->priv->book_view));    
 }
 
 
+static void
+set_crop (SimpleScan *ui, const gchar *crop_name)
+{
+    Page *page;
+    
+    gtk_widget_set_sensitive (ui->priv->crop_rotate_menuitem, crop_name != NULL);
+
+    if (ui->priv->updating_page_menu)
+        return;
+    
+    page = book_view_get_selected (ui->priv->book_view);
+    if (!page)
+        return;
+    
+    if (!crop_name) {
+        page_set_no_crop (page);
+        return;
+    }
+    
+    page_set_named_crop (page, crop_name);
+}
+
+
+void no_crop_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+no_crop_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_crop (ui, NULL);
+}
+
+
+void custom_crop_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+custom_crop_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_crop (ui, "custom");
+}
+
+
+void four_by_six_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+four_by_six_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_crop (ui, "4x6");
+}
+
+                         
+void legal_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+legal_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_crop (ui, "legal");
+}
+
+                         
+void letter_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+letter_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_crop (ui, "letter");
+}
+
+                         
+void a6_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+a6_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_crop (ui, "A6");
+}
+
+
+void a5_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+a5_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_crop (ui, "A5");
+}
+
+                         
+void a4_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+a4_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_crop (ui, "A4");
+}
+
+
+void crop_rotate_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+crop_rotate_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    Page *page;
+
+    page = book_view_get_selected (ui->priv->book_view);
+    if (!page)
+        return;
+    
+    page_rotate_crop (page);
+}
+
+
+void page_delete_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+page_delete_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    book_delete_page (book_view_get_book (ui->priv->book_view),
+                      book_view_get_selected (ui->priv->book_view));
+}
+
+
+void save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
 save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
@@ -275,19 +517,49 @@ static void
 draw_page (GtkPrintOperation *operation,
            GtkPrintContext   *print_context,
            gint               page_number,
-           SimpleScan                *ui)
+           SimpleScan        *ui)
 {
     cairo_t *context;
+    Book *book;
+    Page *page;
+    GdkPixbuf *image;
+    gboolean is_landscape = FALSE;
 
     context = gtk_print_context_get_cairo_context (print_context);
+   
+    book = book_view_get_book (ui->priv->book_view);
+    page = book_get_page (book, page_number);
 
-    g_signal_emit (G_OBJECT (ui), signals[PRINT], 0, context);
+    /* Rotate to same aspect */
+    if (gtk_print_context_get_width (print_context) > gtk_print_context_get_height (print_context))
+        is_landscape = TRUE;
+    if (page_is_landscape (page) != is_landscape) {
+        cairo_translate (context, gtk_print_context_get_width (print_context), 0);
+        cairo_rotate (context, M_PI_2);
+    }
+   
+    cairo_scale (context,
+                 gtk_print_context_get_dpi_x (print_context) / page_get_dpi (page),
+                 gtk_print_context_get_dpi_y (print_context) / page_get_dpi (page));
 
-    //For some reason can't destroy until job complete
-    //cairo_destroy (context);
+    image = page_get_cropped_image (page);
+    gdk_cairo_set_source_pixbuf (context, image, 0, 0);
+    cairo_paint (context);
+
+    g_object_unref (image);
 }
 
 
+void email_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+email_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    g_signal_emit (G_OBJECT (ui), signals[EMAIL], 0);
+}
+
+
+void print_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
 print_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
@@ -295,11 +567,12 @@ print_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
     GtkPrintOperation *print;
     GtkPrintOperationResult result;
     GError *error = NULL;
-    
+    Book *book;
+   
+    book = book_view_get_book (ui->priv->book_view);
+
     print = gtk_print_operation_new ();
-    gtk_print_operation_set_n_pages (print, 1);
-    gtk_print_operation_set_use_full_page (print, TRUE);
-    // FIXME: Auto portrait, landscape
+    gtk_print_operation_set_n_pages (print, book_get_n_pages (book));
     g_signal_connect (print, "draw-page", G_CALLBACK (draw_page), ui);
 
     result = gtk_print_operation_run (print, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
@@ -309,6 +582,24 @@ print_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 }
 
 
+void next_page_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+next_page_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    book_view_select_next_page (ui->priv->book_view);
+}
+
+
+void prev_page_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+prev_page_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    book_view_select_prev_page (ui->priv->book_view);    
+}
+
+
 static void
 load_device_cache (SimpleScan *ui)
 {
@@ -425,6 +716,7 @@ save_device_cache (SimpleScan *ui)
 }
 
 
+void about_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
 about_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
@@ -488,6 +780,7 @@ quit (SimpleScan *ui)
 }
 
 
+void quit_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
 quit_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
@@ -496,6 +789,7 @@ quit_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
 }
 
 
+gboolean window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui);
 G_MODULE_EXPORT
 gboolean
 window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui)
@@ -514,7 +808,7 @@ ui_load (SimpleScan *ui)
     gchar *device, *document_type;
     gboolean replace_pages;
 
-    builder = gtk_builder_new ();
+    builder = ui->priv->builder = gtk_builder_new ();
     gtk_builder_add_from_file (builder, UI_DIR "simple-scan.ui", &error);
     if (error) {
         g_critical ("Unable to load UI: %s\n", error->message);
@@ -531,14 +825,22 @@ ui_load (SimpleScan *ui)
     ui->priv->scan_button_label = GTK_WIDGET (gtk_builder_get_object (builder, "scan_button_label"));
     ui->priv->continuous_scan_button_label = GTK_WIDGET (gtk_builder_get_object (builder, "continuous_scan_button_label"));
     ui->priv->page_label = GTK_WIDGET (gtk_builder_get_object (builder, "page_label"));
-    ui->priv->actions_box = GTK_WIDGET (gtk_builder_get_object (builder, "actions_box"));
     ui->priv->device_combo = GTK_WIDGET (gtk_builder_get_object (builder, "device_combo"));
     ui->priv->device_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->device_combo));
     ui->priv->mode_combo = GTK_WIDGET (gtk_builder_get_object (builder, "mode_combo"));
     ui->priv->mode_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->mode_combo));
-    ui->priv->replace_pages_check = GTK_WIDGET (gtk_builder_get_object (builder, "replace_pages_check"));
     ui->priv->preview_area = GTK_WIDGET (gtk_builder_get_object (builder, "preview_area"));
     ui->priv->zoom_scale = GTK_WIDGET (gtk_builder_get_object (builder, "zoom_scale"));
+    ui->priv->page_delete_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "page_delete_menuitem"));
+    ui->priv->crop_rotate_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "crop_rotate_menuitem"));
+
+    ui->priv->authorize_dialog = GTK_WIDGET (gtk_builder_get_object (builder, "authorize_dialog"));
+    ui->priv->authorize_label = GTK_WIDGET (gtk_builder_get_object (builder, "authorize_label"));
+    ui->priv->username_entry = GTK_WIDGET (gtk_builder_get_object (builder, "username_entry"));
+    ui->priv->password_entry = GTK_WIDGET (gtk_builder_get_object (builder, "password_entry"));
+   
+    ui->priv->preferences_dialog = GTK_WIDGET (gtk_builder_get_object (builder, "preferences_dialog"));
+    ui->priv->replace_pages_check = GTK_WIDGET (gtk_builder_get_object (builder, "replace_pages_check"));
 
     renderer = gtk_cell_renderer_text_new();
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->device_combo), renderer, TRUE);
@@ -569,7 +871,17 @@ ui_load (SimpleScan *ui)
     if (error)
         g_error_free (error);
     else
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ui->priv->replace_pages_check), replace_pages);                                  
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ui->priv->replace_pages_check), replace_pages);
+    
+    ui->priv->book_view = book_view_new ();
+    g_signal_connect (ui->priv->book_view, "page-selected", G_CALLBACK (page_selected_cb), ui);
+    book_view_set_widget (ui->priv->book_view, ui->priv->preview_area,
+                          GTK_WIDGET (gtk_builder_get_object (builder, "page_menu")));
+    gtk_range_set_adjustment (GTK_RANGE (ui->priv->zoom_scale),
+                              book_view_get_zoom_adjustment (ui->priv->book_view));
+    
+    /* Start with a reasonable size */
+    gtk_window_resize (GTK_WINDOW (ui->priv->window), 600, 400);
 }
 
 
@@ -580,17 +892,24 @@ ui_new ()
 }
 
 
-GtkWidget *
-ui_get_preview_widget (SimpleScan *ui)
+void
+ui_set_book (SimpleScan *ui, Book *book)
 {
-    return ui->priv->preview_area;
+    book_view_set_book (ui->priv->book_view, book);
 }
 
 
 void
-ui_set_zoom_adjustment (SimpleScan *ui, GtkAdjustment *adjustment)
+ui_set_selected_page (SimpleScan *ui, Page *page)
 {
-    gtk_range_set_adjustment (GTK_RANGE (ui->priv->zoom_scale), adjustment);
+    book_view_select_page (ui->priv->book_view, page);
+}
+
+
+Page *
+ui_get_selected_page (SimpleScan *ui)
+{
+    return book_view_get_selected (ui->priv->book_view);
 }
 
 
@@ -614,13 +933,15 @@ ui_set_scanning (SimpleScan *ui, gboolean scanning)
                              /* Label on continuous scan button */
                              _("_Continuous Scan"));
     }
+
+    gtk_widget_set_sensitive (ui->priv->page_delete_menuitem, !scanning);
 }
 
 
 void
 ui_set_have_scan (SimpleScan *ui, gboolean have_scan)
 {
-    gtk_widget_set_sensitive (ui->priv->actions_box, have_scan);
+    //FIXME: gtk_widget_set_sensitive (ui->priv->actions_box, have_scan);
 }
 
 
@@ -651,44 +972,6 @@ ui_start (SimpleScan *ui)
 
 /* Generated with glib-genmarshal */
 static void
-g_cclosure_user_marshal_VOID__POINTER_POINTER (GClosure     *closure,
-                                               GValue       *return_value G_GNUC_UNUSED,
-                                               guint         n_param_values,
-                                               const GValue *param_values,
-                                               gpointer      invocation_hint G_GNUC_UNUSED,
-                                               gpointer      marshal_data)
-{
-    typedef void (*GMarshalFunc_VOID__POINTER_POINTER) (gpointer     data1,
-                                                                     gpointer     arg_1,
-                                                                     gpointer     arg_2,
-                                                                     gpointer     data2);
-    register GMarshalFunc_VOID__POINTER_POINTER callback;
-    register GCClosure *cc = (GCClosure*) closure;
-    register gpointer data1, data2;
-    
-    g_return_if_fail (n_param_values == 3);
-    
-    if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-        data1 = closure->data;
-        data2 = g_value_peek_pointer (param_values + 0);
-    }
-    else
-    {
-        data1 = g_value_peek_pointer (param_values + 0);
-        data2 = closure->data;
-    }
-    callback = (GMarshalFunc_VOID__POINTER_POINTER) (marshal_data ? marshal_data : cc->callback);
-    
-    callback (data1,
-              g_value_get_pointer (param_values + 1),
-              g_value_get_pointer (param_values + 2),
-              data2);
-}
-
-
-/* Generated with glib-genmarshal */
-void
 g_cclosure_user_marshal_VOID__STRING_STRING_BOOLEAN_BOOLEAN (GClosure     *closure,
                                                              GValue       *return_value G_GNUC_UNUSED,
                                                              guint         n_param_values,
@@ -748,22 +1031,6 @@ ui_class_init (SimpleScanClass *klass)
                       NULL, NULL,
                       g_cclosure_marshal_VOID__VOID,
                       G_TYPE_NONE, 0);
-    signals[ROTATE_LEFT] =
-        g_signal_new ("rotate-left",
-                      G_TYPE_FROM_CLASS (klass),
-                      G_SIGNAL_RUN_LAST,
-                      G_STRUCT_OFFSET (SimpleScanClass, rotate_left),
-                      NULL, NULL,
-                      g_cclosure_marshal_VOID__VOID,
-                      G_TYPE_NONE, 0);
-    signals[ROTATE_RIGHT] =
-        g_signal_new ("rotate-right",
-                      G_TYPE_FROM_CLASS (klass),
-                      G_SIGNAL_RUN_LAST,
-                      G_STRUCT_OFFSET (SimpleScanClass, rotate_right),
-                      NULL, NULL,
-                      g_cclosure_marshal_VOID__VOID,
-                      G_TYPE_NONE, 0);
     signals[SAVE] =
         g_signal_new ("save",
                       G_TYPE_FROM_CLASS (klass),
@@ -772,14 +1039,14 @@ ui_class_init (SimpleScanClass *klass)
                       NULL, NULL,
                       g_cclosure_marshal_VOID__POINTER,
                       G_TYPE_NONE, 1, G_TYPE_POINTER);
-    signals[PRINT] =
-        g_signal_new ("print",
+    signals[EMAIL] =
+        g_signal_new ("email",
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
-                      G_STRUCT_OFFSET (SimpleScanClass, print),
+                      G_STRUCT_OFFSET (SimpleScanClass, email),
                       NULL, NULL,
-                      g_cclosure_marshal_VOID__POINTER,
-                      G_TYPE_NONE, 1, G_TYPE_POINTER);
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE, 0);
     signals[QUIT] =
         g_signal_new ("quit",
                       G_TYPE_FROM_CLASS (klass),
